@@ -121,7 +121,6 @@ class Client(APIClient, ConfigMixin, CacheMixin, Operators):
         analytics.write_key = decode(self.mixpanel_write_key).decode("utf-8")
 
     def _get_token(self):
-        # TODO: either use cache or keyring package
         token = getpass.getpass(f"Activation token:")
         return token
 
@@ -133,7 +132,7 @@ class Client(APIClient, ConfigMixin, CacheMixin, Operators):
     def create_dataset(self, dataset_id: str, schema: Optional[Dict] = None):
         """
         A dataset can store documents to be searched, retrieved, filtered and aggregated (similar to Collections in MongoDB, Tables in SQL, Indexes in ElasticSearch).
-        A powerful and core feature of VecDB is that you can store both your metadata and vectors in the same document. When specifying the schema of a dataset and inserting your own vector use the suffix (ends with) "_vector_" for the field name, and specify the length of the vector in dataset_schema. \n
+        A powerful and core feature of RelevanceAI is that you can store both your metadata and vectors in the same document. When specifying the schema of a dataset and inserting your own vector use the suffix (ends with) "_vector_" for the field name, and specify the length of the vector in dataset_schema. \n
 
         For example:
 
@@ -157,7 +156,7 @@ class Client(APIClient, ConfigMixin, CacheMixin, Operators):
                 "product_text_chunkvector_" : 1024
             }
 
-        You don't have to specify the schema of every single field when creating a dataset, as VecDB will automatically detect the appropriate data type for each field (vectors will be automatically identified by its "_vector_" suffix). Infact you also don't always have to use this endpoint to create a dataset as /datasets/bulk_insert will infer and create the dataset and schema as you insert new documents. \n
+        You don't have to specify the schema of every single field when creating a dataset, as RelevanceAI will automatically detect the appropriate data type for each field (vectors will be automatically identified by its "_vector_" suffix). Infact you also don't always have to use this endpoint to create a dataset as /datasets/bulk_insert will infer and create the dataset and schema as you insert new documents. \n
 
         Note:
 
@@ -165,8 +164,6 @@ class Client(APIClient, ConfigMixin, CacheMixin, Operators):
             - "_id" is reserved as the key and id of a document.
             - Once a schema is set for a dataset it cannot be altered. If it has to be altered, utlise the copy dataset endpoint.
 
-        For more information about vectors check out the 'Vectorizing' section, services.search.vector or out blog at https://relevance.ai/blog. For more information about chunks and chunk vectors check out datasets.search.chunk.
-
         Parameters
         ----------
         dataset_id: str
@@ -265,52 +262,6 @@ class Client(APIClient, ConfigMixin, CacheMixin, Operators):
             text_fields=text_fields,
         )
 
-    ### Clustering
-
-    @track
-    def ClusterOps(
-        self,
-        dataset_id: str,
-        vector_fields: list,
-        alias: str,
-        model=None,
-        **kwargs,
-    ):
-        from relevanceai.operations_new.cluster.ops import ClusterOps
-
-        return ClusterOps(
-            credentials=self.credentials,
-            dataset_id=dataset_id,
-            vector_fields=vector_fields,
-            alias=alias,
-            model=model,
-            **kwargs,
-        )
-
-    @track
-    def SubClusterOps(
-        self,
-        credentials,
-        alias,
-        dataset,
-        model,
-        vector_fields: list,
-        parent_field: str,
-    ):
-        """
-        Sub Cluster Ops.
-        """
-        from relevanceai.operations.cluster.sub import SubClusterOps
-
-        return SubClusterOps(
-            credentials=self.credentials,
-            alias=alias,
-            dataset=dataset,
-            model=model,
-            vector_fields=vector_fields,
-            parent_field=parent_field,
-        )
-
     def _set_logger_to_verbose(self):
         # Use this for debugging
         self.config["logging.logging_level"] = "INFO"
@@ -460,16 +411,6 @@ class Client(APIClient, ConfigMixin, CacheMixin, Operators):
 
     docs = references
 
-    def search_app(self, dataset_id: Optional[str] = None):
-        if dataset_id is not None:
-            self.print_search_dashboard_url(dataset_id)
-        elif hasattr(self, "_dataset_id"):
-            self.print_search_dashboard_url(self._dataset_id)
-        elif hasattr(self, "dataset_id"):
-            self.print_search_dashboard_url(self.dataset_id)
-        else:
-            print("You can build your search app at https://cloud.relevance.ai")
-
     @added(version="1.1.3")
     @track
     def search_datasets(self, query: str):
@@ -478,62 +419,6 @@ class Client(APIClient, ConfigMixin, CacheMixin, Operators):
         """
         return [x for x in self.list_datasets()["datasets"] if query in x]
 
-    @added(version="2.1.3")
-    @beta
-    def list_cluster_reports(self):
-        """
-
-        List all cluster reports.
-
-        .. code-block::
-
-            from relevanceai import Client
-            client = Client()
-            client.list_cluster_reports()
-
-        """
-        return pd.DataFrame(self.reports.clusters.list()["results"])
-
-    @added(version="2.1.3")
-    @beta
-    @track
-    def delete_cluster_report(self, cluster_report_id: str):
-        """
-
-        Delete Cluster Report
-
-        .. code-block::
-
-            from relevanceai import Client
-            client = Client()
-            client.delete_cluster_report("cluster_id_goes_here")
-
-        """
-        return self.reports.clusters.delete(cluster_report_id)
-
-    @added(version="2.1.3")
-    @beta
-    @track
-    def store_cluster_report(self, report_name: str, report: dict):
-        """
-
-        Store the cluster data.
-
-        .. code-block::
-
-            from relevanceai import Client
-            client = Client()
-            client.store_cluster_report("sample", {"value": 3})
-
-        """
-        response: dict = self.reports.clusters.create(
-            name=report_name, report=self.json_encoder(report)
-        )
-        print(
-            f"You can now access your report at https://cloud.relevance.ai/report/cluster/{self.region}/{response['_id']}"
-        )
-        return response
-
     def disable_analytics_tracking(self):
         """Disable analytics tracking if you would prefer not to send usage
         data to improve the product. Analytics allows us to improve your experience
