@@ -1,10 +1,12 @@
-from relevanceai.utils.decorators.analytics import track
+import pandas as pd
+
 from relevanceai.dataset.read import Read
+from relevanceai.utils.decorators.analytics import track
 
 
 class DictExport(Read):
     @track
-    def to_dict(self, orient: str = "records"):
+    def to_dict(self, orient: str = "records", **kwargs):
         """
         Returns the raw list of dicts from Relevance AI
 
@@ -29,7 +31,10 @@ class DictExport(Read):
 
             dict = df.to_dict(orient="records")
         """
+        documents = self.get_all_documents(**kwargs)
+
         if orient == "records":
-            return self.get_all_documents()
+            return documents
         else:
             raise NotImplementedError
+            dataframe = pd.DataFrame(documents)
