@@ -122,7 +122,7 @@ class Read(ClusterRead):
 
             length, width = df.shape
         """
-        schema = self.datasets.schema(self.dataset_id)
+        schema = self.schema
         n_documents = self.get_number_of_documents(dataset_id=self.dataset_id)
         return (n_documents, len(schema))
 
@@ -604,7 +604,7 @@ class Read(ClusterRead):
         number_of_documents: int = 20,
         filters: Optional[list] = None,
         cursor: str = None,
-        batch_size: int = 1000,
+        chunksize: int = 1000,
         sort: Optional[list] = None,
         select_fields: Optional[list] = None,
         include_vector: bool = True,
@@ -625,7 +625,7 @@ class Read(ClusterRead):
             Fields to include in the search results, empty array/list means all fields.
         cursor: string
             Cursor to paginate the document retrieval
-        batch_size: int
+        chunksize: int
             Number of documents to retrieve per iteration
         include_vector: bool
             Include vectors in the search results
@@ -643,7 +643,7 @@ class Read(ClusterRead):
             number_of_documents=number_of_documents,
             filters=filters,
             cursor=cursor,
-            batch_size=batch_size,
+            chunksize=chunksize,
             sort=sort,
             select_fields=select_fields,
             include_vector=include_vector,
@@ -657,7 +657,10 @@ class Read(ClusterRead):
         """
         Store Metadata
         """
-        return self.datasets.metadata(self.dataset_id)["results"]
+        try:
+            return self.datasets.metadata(self.dataset_id)["results"]
+        except:
+            return {}
 
     @property
     def metadata(self):
@@ -705,7 +708,11 @@ class Read(ClusterRead):
 
     @track
     def chunk_dataset(
-        self, select_fields: List = None, chunksize: int = 100, filters: list = None
+        self,
+        select_fields: List = None,
+        chunksize: int = 100,
+        filters: list = None,
+        after_id: list = None,
     ):
         """
 
@@ -736,6 +743,7 @@ class Read(ClusterRead):
             filters=filters,
             select_fields=select_fields,
             include_after_id=True,
+            after_id=after_id,
         )
         number_of_documents = self.get_number_of_documents(
             self.dataset_id, filters=filters
@@ -754,7 +762,7 @@ class Read(ClusterRead):
                 yield docs["documents"]
                 docs = self.get_documents(
                     number_of_documents=chunksize,
-                    include_cursor=True,
+                    include_cursor=False,
                     after_id=docs["after_id"],
                     filters=filters,
                     select_fields=select_fields,
@@ -784,7 +792,7 @@ class Read(ClusterRead):
             ds.list_vector_fields()
 
         """
-        schema = self.datasets.schema(self.dataset_id)
+        schema = self.schema
         return [
             k for k in schema.keys() if k.endswith("_vector_") and "_cluster_" not in k
         ]
@@ -841,3 +849,12 @@ class Read(ClusterRead):
             page=page,
             asc=asc,
         )
+
+    def get_settings(self):
+        """
+        Get the settings in dataset
+        """
+        return self.datasets.get_settings(dataset_id=self.dataset_id)
+
+    def list_cluster_fields(self):
+        return [x for x in self.schema if "_cluster_" in x and x.count(".") >= 2]
