@@ -1,6 +1,7 @@
 """
 Batch Cluster Operations
 """
+from relevanceai.constants.links import EXPLORER_APP_LINK
 from relevanceai.operations_new.cluster.batch.transform import BatchClusterTransform
 from relevanceai.operations_new.ops_base import OperationAPIBase
 from relevanceai.operations_new.cluster.ops import ClusterOps
@@ -8,6 +9,8 @@ from relevanceai.operations_new.cluster.batch.models.base import BatchClusterMod
 from relevanceai.dataset import Dataset
 from typing import Any
 
+from relevanceai.operations_new.ops_run import PullTransformPush
+
 
 class BatchClusterOps(BatchClusterTransform, ClusterOps):
     """Batch Clustering related Operations"""
@@ -55,25 +58,53 @@ class BatchClusterOps(BatchClusterTransform, ClusterOps):
 
         self.alias = self._get_alias(alias)
 
-    def run(self, dataset: Dataset, filters: list = None):
+    def fit(self, chunk):
+        vectors = self.get_field_across_documents(
+            self.vector_fields[0], chunk, missing_treatment="skip"
+        )
+        if hasattr(self.model, "n_clusters"):
+            if len(vectors) < self.model.n_clusters:
+                return chunk
+        self.model.partial_fit(vectors)
+        return chunk
+
+    def run(
+        self, dataset: Dataset, filters: list = None, chunksize: int = 500, **kwargs
+    ):
         """
         Run batch clustering
         """
-        # TODO:
-        # Avoid looping through dataset twice
-        print("Fitting...")
-        for chunk in dataset.chunk_dataset(
-            select_fields=self.vector_fields, filters=filters
-        ):
-            vectors = self.get_field_across_documents(
-                self.vector_fields[0], chunk, missing_treatment="skip"
-            )
-            self.model.partial_fit(vectors)
+        from tqdm.auto import tqdm
 
-        print("Predicting...")
-        for chunk in dataset.chunk_dataset(select_fields=self.vector_fields):
-            # Provide a chunk
-            chunk = self.transform(chunk)
-            results = dataset.upsert_documents(chunk)
+        tqdm.write("\nFitting Model...")
+        ptp = PullTransformPush(
+            dataset=dataset,
+            func=self.fit,
+            pull_chunksize=chunksize,
+            push_chunksize=chunksize,
+            filters=filters,
+            select_fields=self.vector_fields,
+            show_progress_bar=True,
+            **kwargs
+        )
+        ptp.run()
 
+        kwargs.pop("transform_chunksize")
+
+        tqdm.write("\nPredicting Documents...")
+        ptp = PullTransformPush(
+            dataset=dataset,
+            func=self.transform,
+            pull_chunksize=chunksize,
+            transform_chunksize=chunksize,
+            push_chunksize=chunksize,
+            filters=filters,
+            select_fields=self.vector_fields,
+            show_progress_bar=True,
+            **kwargs
+        )
+        ptp.run()
+
+        tqdm.write("\nConfigure your new explore app below:")
+        tqdm.write(EXPLORER_APP_LINK.format(dataset.dataset_id))
         return
