@@ -20,11 +20,104 @@ from tqdm.auto import tqdm
 from typing import Any, Dict, List, Optional
 from datetime import datetime
 from relevanceai.dataset.write import Write
+
 from relevanceai.utils.decorators.analytics import track
 from relevanceai.constants import EXPLORER_APP_LINK
 
 
+def get_ptp_args() -> List[str]:
+    """
+    Returns all arguments in the PullTransformPush.__init__ func
+    """
+
+    from relevanceai.operations_new.ops_run import PullTransformPush, arguments
+
+    return arguments(PullTransformPush)
+
+
 class Operations(Write):
+    @staticmethod
+    def _get_filters(input_fields: List[str], output_fields: List[str]):
+        """
+        Creates the filters necessary to search all documents
+        within a dataset that contain fields specified in "input_fields"
+        but do not contain the fields specified in "output_fields"
+
+        e.g.
+        fields = ["text", "title"]
+        vector_fields = ["text_use_vector_", "title_use_vector_"]
+
+        we want to search the dataset where:
+        ("text" * ! "text_use_vector_") + ("title" * ! "title_use_vector_")
+
+        Since the current implementation of filtering only accounts for CNF and not DNF boolean logic,
+        We must use boolean algebra here to obtain the CNF from a DNF expression.
+
+        CNF = Conjunctive Normal Form (Sum of Products)
+        DNF = Disjunctive Normal Form (Product of Sums)
+
+        This means converting the above to:
+        ("text" + "title") * ("text" + ! "title_use_vector_") *
+        (! "text_use_vector_" + "title") * (! "text_use_vector_" + ! "title_use_vector_")
+
+        Arguments:
+            input_fields: List[str]
+                A list of fields within the dataset
+
+            output_fields: List[str]
+                A list of output_fields, created from the operation.
+                These would be present in processed documents
+
+        Returns:
+            filters: List[Dict[str, Any]]
+                A list of filters.
+        """
+
+        if len(input_fields) > 1:
+            iters = len(input_fields) ** 2
+
+            filters: list = []
+            for i in range(iters):
+                binary_array = [character for character in str(bin(i))][2:]
+                mixed_mask = ["0"] * (
+                    len(input_fields) - len(binary_array)
+                ) + binary_array
+                mask = [int(value) for value in mixed_mask]
+                # Creates a binary mask the length of fields provided
+                # for two fields, we need 4 iters, going over [(0, 0), (1, 0), (0, 1), (1, 1)]
+
+                condition_value = [
+                    {
+                        "field": field if mask[index] else vector_field,
+                        "filter_type": "exists",
+                        "condition": "==" if mask[index] else "!=",
+                        "condition_value": "",
+                    }
+                    for index, (field, vector_field) in enumerate(
+                        zip(input_fields, output_fields)
+                    )
+                ]
+                filters += [{"filter_type": "or", "condition_value": condition_value}]
+
+        else:  # Special Case when only 1 field is provided
+            condition_value = [
+                {
+                    "field": input_fields[0],
+                    "filter_type": "exists",
+                    "condition": "==",
+                    "condition_value": " ",
+                },
+                {
+                    "field": output_fields[0],
+                    "filter_type": "exists",
+                    "condition": "!=",
+                    "condition_value": " ",
+                },
+            ]
+            filters = condition_value
+
+        return filters
+
     @track
     def reduce_dims(
         self,
@@ -37,6 +130,7 @@ class Operations(Write):
         filters: Optional[list] = None,
         chunksize: Optional[int] = 100,
         output_field: str = None,
+        **kwargs,
     ):
         """It takes a list of fields, a list of models, a list of filters, and a chunksize, and then runs
         the DimReductionOps class on the documents in the dataset
@@ -71,12 +165,19 @@ class Operations(Write):
             output_field=output_field,
         )
 
+        filters = [] if filters is None else filters
+        filters += Operations._get_filters(
+            input_fields=vector_fields,
+            output_fields=[ops.model.vector_name(vector_fields, output_field)],
+        )
+
         res = ops.run(
             dataset=self,
             select_fields=vector_fields,
             chunksize=chunksize,
             filters=filters,
             batched=batched,
+            **kwargs,
         )
 
         return ops
@@ -88,8 +189,9 @@ class Operations(Write):
         batched: bool = True,
         models: Optional[List[Any]] = None,
         filters: Optional[list] = None,
-        chunksize: Optional[int] = 20,
+        chunksize: Optional[int] = None,
         output_fields: list = None,
+        **kwargs,
     ):
         """It takes a list of fields, a list of models, a list of filters, and a chunksize, and then it runs
         the VectorizeOps function on the documents in the database
@@ -122,15 +224,19 @@ class Operations(Write):
         )
 
         filters = [] if filters is None else filters
-        filters += ops._get_base_filters()
+        filters += Operations._get_filters(
+            input_fields=fields, output_fields=ops.vector_fields
+        )
+
         res = ops.run(
             dataset=self,
             select_fields=fields,
             filters=filters,
             batched=batched,
             chunksize=chunksize,
+            output_fields=output_fields,
+            **kwargs,
         )
-
         return ops
 
     @track
@@ -141,6 +247,7 @@ class Operations(Write):
         batched: Optional[bool] = True,
         filters: Optional[list] = None,
         chunksize: Optional[int] = 20,
+        **kwargs,
     ):
         """It takes a list of fields, a list of models, a list of filters, and a chunksize, and then it runs
         the VectorizeOps function on the documents in the database
@@ -172,7 +279,9 @@ class Operations(Write):
         )
 
         filters = [] if filters is None else filters
-        filters += ops._get_base_filters()
+        filters += Operations._get_filters(
+            input_fields=fields, output_fields=ops.vector_fields
+        )
 
         res = ops.run(
             dataset=self,
@@ -180,6 +289,7 @@ class Operations(Write):
             filters=filters,
             batched=batched,
             chunksize=chunksize,
+            **kwargs,
         )
 
         return ops
@@ -330,6 +440,7 @@ class Operations(Write):
         batched: bool = False,
         filters: Optional[list] = None,
         chunksize: Optional[int] = 100,
+        **kwargs,
     ):
         """
         This function splits the text in the `text_field` into sentences and stores the sentences in
@@ -357,12 +468,15 @@ class Operations(Write):
             output_field=output_field,
         )
 
+        filters = [] if filters is None else filters
+
         res = ops.run(
             dataset=self,
             select_fields=text_fields,
             batched=batched,
             filters=filters,
             chunksize=chunksize,
+            **kwargs,
         )
 
         return ops
@@ -374,10 +488,10 @@ class Operations(Write):
         model: Optional[Any] = None,
         alias: Optional[str] = None,
         model_kwargs: Optional[Dict[str, Any]] = None,
-        chunksize: Optional[int] = 100,
+        chunksize: Optional[int] = 128,
         filters: Optional[list] = None,
         batched: Optional[bool] = False,
-        include_cluster_report: bool = True,
+        include_cluster_report: bool = False,
         **kwargs,
     ):
         """`cluster` is a function that takes in a list of vector fields, a model, an alias, a list of
@@ -437,6 +551,7 @@ class Operations(Write):
             credentials=self.credentials,
             dataset_id=self.dataset_id,
             model_kwargs=model_kwargs,
+            include_cluster_report=include_cluster_report,
             **kwargs,
         )
 
@@ -450,9 +565,7 @@ class Operations(Write):
             chunksize=chunksize,
             filters=filters,
         )
-        # TODO: Create the cluster report
-        if include_cluster_report:
-            pass
+        print()
         print(
             f"""You can now utilise the ClusterOps object using the below:
 
@@ -462,9 +575,8 @@ class Operations(Write):
         dataset_id='{self.dataset_id}'
     )"""
         )
-
-        print("Configure your new cluster app below:")
         print()
+        print("Configure your new explore app below:")
         print(EXPLORER_APP_LINK.format(self.dataset_id))
         return ops
 
@@ -501,12 +613,21 @@ class Operations(Write):
         model: Any = None,
         alias: Optional[str] = None,
         filters: Optional[list] = None,
-        include_cluster_report: bool = True,
-        model_kwargs: dict = None,
+        model_kwargs: Dict = None,
+        chunksize: int = 128,
         **kwargs,
     ):
         from relevanceai.operations_new.cluster.batch.ops import BatchClusterOps
 
+        n_clusters = ({} if model_kwargs is None else model_kwargs).get("n_clusters", 8)
+        if kwargs.get("transform_chunksize") is None:
+            kwargs["transform_chunksize"] = max(n_clusters * 5, 128)
+
+        run_kwargs = {}
+        for key in get_ptp_args():
+            if key in kwargs:
+                run_kwargs[key] = kwargs.pop(key)
+
         cluster_ops = BatchClusterOps(
             model=model,
             alias=alias,
@@ -515,10 +636,9 @@ class Operations(Write):
             **kwargs,
         )
 
-        if filters is not None:
-            filters = cluster_ops._get_filters(filters, vector_fields)
+        filters = cluster_ops._get_filters(filters, vector_fields)  # type: ignore
 
-        cluster_ops.run(self, filters=filters)
+        cluster_ops.run(self, filters=filters, chunksize=chunksize, **run_kwargs)
 
         return cluster_ops
 
@@ -532,8 +652,9 @@ class Operations(Write):
         sensitivity: float = 0,
         filters: Optional[list] = None,
         output_fields: list = None,
-        chunksize: int = 100,
+        chunksize: int = 128,
         batched: bool = True,
+        **kwargs,
     ):
         """
         Extract sentiment from the dataset
@@ -554,12 +675,15 @@ class Operations(Write):
             output_fields=output_fields,
             sensitivity=sensitivity,
         )
+        filters = [] if filters is None else filters
+        filters += SentimentOps._get_filters(text_fields, model_name)
         ops.run(
             self,
             filters=filters,
             select_fields=text_fields,
             chunksize=chunksize,
             batched=batched,
+            **kwargs,
         )
         return ops
 
@@ -573,6 +697,7 @@ class Operations(Write):
         min_score: float = 0.3,
         batched: bool = True,
         refresh: bool = False,
+        **kwargs,
     ):
         """
         Extract an emotion.
@@ -589,6 +714,7 @@ class Operations(Write):
         """
         from relevanceai.operations_new.emotion.ops import EmotionOps
 
+        filters = [] if filters is None else filters
         ops = EmotionOps(
             credentials=self.credentials,
             text_fields=text_fields,
@@ -596,6 +722,15 @@ class Operations(Write):
             output_fields=output_fields,
             min_score=min_score,
         )
+        filters += [
+            {
+                "field": text_field,
+                "filter_type": "exists",
+                "condition": ">=",
+                "condition_value": " ",
+            }
+            for text_field in text_fields
+        ]
         ops.run(
             self,
             filters=filters,
@@ -604,6 +739,7 @@ class Operations(Write):
             batched=batched,
             output_fields=output_fields,
             refresh=refresh,
+            **kwargs,
         )
         return ops
 
@@ -614,6 +750,7 @@ class Operations(Write):
         output_fields: Optional[List[str]] = None,
         filters: Optional[list] = None,
         refresh: bool = False,
+        **kwargs,
     ):
         """
         Apply a transformers pipeline generically.
@@ -643,6 +780,7 @@ class Operations(Write):
             select_fields=text_fields,
             output_fields=output_fields,
             refresh=refresh,
+            **kwargs,
         )
         return ops
 
@@ -655,6 +793,7 @@ class Operations(Write):
         filters: Optional[list] = None,
         batched: Optional[bool] = None,
         chunksize: Optional[int] = None,
+        **kwargs,
     ):
 
         from relevanceai.operations_new.scaling.ops import ScaleOps
@@ -676,6 +815,7 @@ class Operations(Write):
             chunksize=chunksize,
             filters=filters,
             select_fields=vector_fields,
+            **kwargs,
         )
         return ops
 
@@ -708,6 +848,11 @@ class Operations(Write):
             **kwargs,
         )
 
+        run_kwargs = {}
+        for key in get_ptp_args():
+            if key in kwargs:
+                run_kwargs[key] = kwargs.pop(key)
+
         # Building an infinitely hackable SDK
 
         # Add filters and select fields
@@ -747,6 +892,7 @@ class Operations(Write):
             self,
             filters=filters,
             select_fields=select_fields,
+            **run_kwargs,
         )
         print(
             f"""You can now utilise the ClusterOps object based on subclustering.
@@ -833,6 +979,7 @@ class Operations(Write):
         lemmatize: bool = False,
         filters: list = None,
         replace_words: dict = None,
+        **kwargs,
     ):
         """
         Cleans text for you!
@@ -854,6 +1001,7 @@ class Operations(Write):
             remove_stopword=remove_stopwords,
             lemmatize=lemmatize,
             replace_words=replace_words,
+            **kwargs,
         )
 
         print("🥸 A clean house is a sign of no Internet connection.")
@@ -876,6 +1024,7 @@ class Operations(Write):
         filters: list = None,
         chunksize: int = 1000,
         refresh: bool = False,
+        **kwargs,
     ):
         from relevanceai.operations_new.processing.text.count.ops import CountTextOps
 
@@ -893,6 +1042,7 @@ class Operations(Write):
             filters=filters,
             batched=True,
             refresh=refresh,
+            **kwargs,
         )
         return ops
 
@@ -1036,6 +1186,11 @@ class Operations(Write):
         stop_words: list = None,
         filters: list = None,
         batched: bool = True,
+        use_maxsum: bool = False,
+        nr_candidates: int = 20,
+        use_mmr=True,
+        diversity=0.7,
+        **kwargs,
     ):
         """
         Extract the keyphrases of a text field and output and store it into
@@ -1053,6 +1208,10 @@ class Operations(Write):
             output_fields=output_fields,
             stop_words=stop_words,
             max_keywords=max_keywords,
+            nr_candidates=nr_candidates,
+            use_maxsum=use_maxsum,
+            use_mmr=use_mmr,
+            diversity=diversity,
         )
         ops.run(
             self,
@@ -1061,6 +1220,7 @@ class Operations(Write):
             filters=filters,
             select_fields=fields,
             output_fields=output_fields,
+            **kwargs,
         )
         return ops
 
@@ -1198,3 +1358,79 @@ class Operations(Write):
         )
 
         return ops
+
+    def extract_ner(
+        self,
+        fields: list,
+        model_id: str = None,
+        output_fields: list = None,
+        chunksize: int = 20,
+        filters: list = None,
+        refresh: bool = False,
+    ):
+        """
+        Extract NER
+        """
+        from relevanceai.operations_new.processing.text.ner.ops import ExtractNEROps
+
+        if model_id is None:
+            model_id = "dslim/bert-base-NER"
+        ops = ExtractNEROps(
+            credentials=self.credentials,
+            fields=fields,
+            model_id=model_id,
+            output_fields=output_fields,
+        )
+
+        ops.run(
+            self,
+            batched=True,
+            chunksize=chunksize,
+            filters=filters,
+            select_fields=fields,
+            output_fields=output_fields,
+            refresh=refresh,
+        )
+
+        return ops
+
+    def tag_text(
+        self,
+        fields: list,
+        model_id: str = None,
+        labels: list = None,
+        output_fields: list = None,
+        chunksize: int = 20,
+        minimum_score: float = 0.2,
+        maximum_number_of_labels: int = 5,
+        filters: list = None,
+        refresh: bool = False,
+        **kwargs,
+    ):
+        """
+        Tag Text
+        """
+        from relevanceai.operations_new.text_tagging.ops import TextTagOps
+
+        ops = TextTagOps(
+            credentials=self.credentials,
+            fields=fields,
+            model_id=model_id,
+            output_fields=output_fields,
+            minimum_score=minimum_score,
+            maximum_number_of_labels=maximum_number_of_labels,
+            labels=labels,
+        )
+
+        ops.run(
+            self,
+            batched=True,
+            chunksize=chunksize,
+            filters=filters,
+            select_fields=fields,
+            output_fields=output_fields,
+            refresh=refresh,
+            **kwargs,
+        )
+
+        return ops
