@@ -1,9 +1,9 @@
 from typing import Any, List
 
 from relevanceai.operations_new.vectorize.ops import VectorizeOps
-from relevanceai.operations_new.vectorize.image.base import VectorizeImageBase
+from relevanceai.operations_new.vectorize.image.transform import VectorizeImageTransform
 
 
-class VectorizeImageOps(VectorizeOps, VectorizeImageBase):
+class VectorizeImageOps(VectorizeOps, VectorizeImageTransform):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
